/*******************************************************************\
Perpetual Calendar
A Gregorian Calendar for Ti-89
by Jonathan Kotta

This program displays a calendar for a given month and year.

I wrote this just as a programming exercise, because I wanted a
better calendar program (i.e. one that was fast and simple and
took command line arguments), and the other C calendars I found we're
wrong.

It seems to me from looking at the source for other calendars that 
nobody really knows what they're doing; there are no comments at all
in the complex algorthms that calculate the calendar.  This leads me
to believe that the programmers just ported the code without 
understanding it (not to mention the errors).  So while my code may 
be ugly, I at least know what I'm doing.

How the calendar works:
The Gregorian Calendar is periodic with a period of 400 years.  In 
addition, it is semi-periodic with a period of 28 years:
((7 days in a week)*(4 year leap cycle)=28).
The only time it's thrown off of this 28 year cycle is at the turn 
of the century because centennial years are not leap years.  Here is
how to determine if a year is a leap year or not:

	Years divisible by 400 are ALWAYS leap years.
	Years divisible by 100 are NOT leap years unless divisible by 400.
	Years divisible by 4 are leap years unless divisble by 100.
	Years are not leap years unless divisible by 4.

There are only (7 days of the week)*(leap or normal)=14 different
calendars.  Below is a complicated table of how the Calendar shifts 
through these years.  'Calendar 06' refers to the calendar that has
Jan. 1 fall on (6 - 1) = 5 = Friday (this is how the World Almanac 
Perpetual Calendar is set up).

Line 1 shows how far along in the 28 year cycle we are.
Line 2 shows the sequence of calendars for the 28 year cycle, 
beginning with a year divisible by 400.
Line 3 shows how the second century starts (starts in the 16th year
of the 28 year cycle, but with calendar 06 instead of 13 because it's
not a leap year).
Line 4 shows reverts back to the 28 year cycle.
Line 5 shows how the third century starts.
Line 6 shows how the third century reverts back to the cycle.
Line 7: start of fourth century.
Line 8: fourth century reverts back to the cycle.
After the fourth century, the 400 year cycle repeats.

1:  00,01,02,03,04,05,06,07,08,09,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27	
2:  14,02,03,04,12,07,01,02,10,05,06,07,08,03,04,05,13,01,02,03,11,06,07,01,09,04,05,06 
3:                                              ...,06,(07)
4:            (06),07,01,02,...
5:                                                          ...,04,(05)
6:                        (04),05,06,07,...
7:                                                                      ...,01,(03)
8:                                    (01),03,04,05,...

I got all this data from the World Almanac's Perpetual Calendar, and
I'd say mine is easier to use and quite a bit closer to being 
perpetual than theirs.
\*******************************************************************/

/********************** DEFINES AND INCLUDES ***********************/
#define OPTIMIZE_ROM_CALLS    // Use ROM Call Optimization
#define SAVE_SCREEN           // Save/Restore LCD Contents
#include <tigcclib.h>         // Include All Header Files

//define only one at a time
//1 == make 89 version,  0 == make 92+ version
#define ti89 1

#if ti89
short _ti89;                  // Produce .89Z File
//89 keys
#define Key_Up 337
#define Key_Down 340
#define Key_Right 344
#define Key_Left 338
#define Key_2nd 4096
#define	Key_Diamond 16384
#else	//NOT ti89
short _ti92plus;              // Produce .9xZ File
//92+ keys
#define Key_Up 338
#define Key_Down 344
#define Key_Right 340
#define Key_Left 337
#define Key_2nd 4096
#define	Key_Diamond 8192
#endif	//NOT ti89

#define JAN 0
#define DEC 11
#define version "1.0"

/************************ GLOBAL VARIABLES *************************/
//NONE

/************************** MAIN FUNCTION **************************/
void _main(void)
{
//for testing
//int jan_offset;

//data tables are chars to save space
char num_days[12] = {31,28,31,30,31,30,31,31,30,31,30,31};

char month[12][10] = {
"January  ", "February ", "March    ", "April    ", 
"May      ", "June     ", "July     ", "August   ",
"September", "October  ", "November ", "December "};

char offset_table[28] = {6,1,2,3,4,6,0,1,2,4,5,6,0,2,3,4,5,0,1,2,3,5,6,0,1,3,4,5};

//static allows you to save the variables (kind of like globals)
static int cur_month = JAN;
static int cur_year = 2002;
int std_year;
int key = 0;
int offset = 0;
int i;

//for getting command line args
ESI arg_ptr;

char *about_text = "

Perpetual Calendar
  version %s

A Gregorian Calendar
  for TI-89

by Jonathan Kotta
  the_juggernaut@juno.com ";

#if ti89
FontSetSys(F_6x8);
#else	//NOT _ti89
FontSetSys(F_8x10);
#endif	//NOT _ti89

//get passed arguments
if(ArgCount() == 2)
{
	InitArgPtr(arg_ptr);
	if(GetArgType(arg_ptr) == POSINT_TAG)
	{
		cur_month = GetIntArg(arg_ptr) - 1;
		cur_month %= 12;
	}
	if(GetArgType(arg_ptr) == POSINT_TAG)
	{
		cur_year = GetIntArg(arg_ptr);
	}
}

//main loop
while(1)
{
//make sure year is valid
	/*  the Gregorian Calendar took effect Oct. 15, 1582
	any date before that you must use the Julian Calendar
	the upper limit is where I estimate that the Gregorian Calendar
	is off by about a day */
	if(cur_year < 1583)
	{
		cur_year = 1583;
	}
	else if(cur_year > 4800)
	{
		cur_year = 4800;
	}


//determine if year is leap year or normal	
	/*  std_year is a normalized year so that 0 <= std_year < 400
	there's no reason to condense this into an if-else statement,
	the size difference is a few bytes and it's clearer this way  */
	if(!(std_year = cur_year % 400))
	{
		num_days[1] = 29;
	}
	else if(!(std_year % 100))
	{
		num_days[1] = 28;
	}
	else if(!(std_year % 4))
	{
		num_days[1] = 29;
	}
	else
	{
		num_days[1] = 28;
	}

//account for shifts in the 28 year cycle because of non-leap
//centennial years
	if(std_year > 100)
	{
		if(std_year <= 200)
		{
			std_year -= 12;
		}
		else if(std_year <= 300)
		{
			std_year -= 24;
		}
		else if(std_year < 400)
		{
			std_year -= 36;
		}
	}
	
	/* this if-else block does the same thing as the above if statement
	but it's sightly clearer  
	if(std_year <= 100)
	{
		std_year = std_year;
	}
	else if((std_year > 100)&&(std_year <= 200))
	{
		std_year -= 12;
	}
	else if((std_year > 200)&&(std_year <= 300))
	{
		std_year -= 12*2;
	}
	else if((std_year > 300)&&(std_year < 400))
	{
		std_year -= 12*3;
	}*/

	//this is now the offset for January
	//offset 0 means Jan 1 is a Sunday
	offset = (int)offset_table[std_year % 28];
	
	//for testing
	//jan_offset = offset;
	
	
	//this changes the offset based on how many days have passed
	//since the beginning of the year
	for(i = 0; i < cur_month; i++)
	{
		offset += num_days[i];
	}
	offset = offset % 7;
	
	
//display calendar
	clrscr();
	printf("\n   %s       %d\n\n", month[cur_month], cur_year);
	printf("   SU MO TU WE TH FR SA\n   ");
	if(offset != 0)
	{
		printf("\n   ");
	}
	for(i = 0; i < offset; i++)
	{
		printf("   ");
	}
	i = 0;
	while(i < (int)num_days[cur_month])
	{
		if(!((i + offset) % 7))
		{
			printf("\n   ");
		}
		printf("%2d ", i + 1);
		i++;
	}
	
	//for testing
	//jan_offset = jan_offset + 1 + 7*((int)num_days[1] - 28);
	//printf("\n%d",jan_offset);
	
//check for key presses and maybe do stuff
	key = GKeyIn(NULL, 0);
	switch(key)
	{
		case Key_Right:
			if(cur_month < DEC)
			{
				cur_month++;
			}
			else
			{
				cur_month = JAN;
				cur_year++;
			}
			break;
		case Key_Left:
			if(cur_month > JAN)
			{
				cur_month--;
			}
			else
			{
				cur_month = DEC;
				cur_year--;
			}
			break;
		case Key_Up:
			cur_year++;
			break;
		case Key_Down:
			cur_year--;
			break;
		case Key_Up + Key_2nd:
			cur_year += 10;
			break;
		case Key_Down + Key_2nd:
			cur_year -= 10;
			break;
		case Key_Up + Key_Diamond:
			cur_year += 100;
			break;
		case Key_Down + Key_Diamond:
			cur_year -= 100;
			break;
		case KEY_F1:
			clrscr();
			printf(about_text, version);
			GKeyIn(NULL, 0);
			break;
		case KEY_ESC:
			return;
		default:
			break;
	} //end switch
} //end while
} //end main


/********************** FUNCTION DEFINITIONS ***********************/
//NONE
/************************** NEXT FUNCTION **************************/

/************************** FUTURE PLANS ***************************\
maybe implement the Julian calendar and the strange year of 1582
\*******************************************************************/
